import os
import logging
import yaml
from fastapi import HTTPException, status
from supabase import create_client, Client, ClientOptions

from ..constants import PARAMETERS_DIR

# Silence the supabase logger
logging.getLogger("httpx").setLevel(logging.CRITICAL)
logging.getLogger("hpack").setLevel(logging.CRITICAL)
logging.getLogger("httpcore").setLevel(logging.CRITICAL)


class Supabase:

    logger = logging.getLogger(__name__)

    def __init__(self):

        with open(os.path.join(PARAMETERS_DIR, 'secrets.yaml')) as f:
            secrets = yaml.safe_load(f)
            self.SUPABASE_URL = secrets['SUPABASE_URL']
            self.SUPABASE_KEY = secrets['SUPABASE_API_KEY']
            self.SUPABASE_TEST_USER_ID = secrets['SUPABASE_TEST_USER_ID']

        self.supabase: Client = create_client(self.SUPABASE_URL, self.SUPABASE_KEY, options=ClientOptions(schema='public'))
        self.logger.debug('Supabase client initialized.')


    def check_balance(self, user_id: str) -> bool:
        """
        Checks if the user has enough 'credit' for generating a new trip.

        Args:
            user_id (str): The ID of the current user.

        Returns:
            bool: True if the balance is positive, False otherwise.
        """
        try:
            # Query the public.credits table to get the user's credits
            response = (
                self.supabase.table("credits")
                .select('*')
                .eq('id', user_id)
                .single()
                .execute()
            )
            # Check if the response was successful and contains data
            if response.data :
                credits = response.data['credit_amount']
                self.logger.debug(f'Credits of user {user_id}: {credits}')

                if credits > 0:
                    self.logger.info(f'Credit balance is positive. Proceeding with trip generation')
                    return True

                self.logger.warning(f'Insufficient balance. Trip generation cannot be granted')
                return False

            # If user or credits not found, raise a 404 error
            raise HTTPException(status_code=404, detail="User data not found.")

        except Exception as exc:
            raise HTTPException(status_code=500, detail="Error retrieving credit balance") from exc


    def decrement_credit_balance(self, user_id: str) -> bool:
        """
        Decrements the user's credit balance by 1.

        Args:
            user_id (str): The ID of the current user.
        """
        try:
            # Query the public.credits table to get the user's current credits
            response = (
                self.supabase.table("credits")
                .select('*')
                .eq('id', user_id)
                .single()
                .execute()
            )
            # Check if the response was successful and contains data
            if response.data :
                current_credits = response.data['credit_amount']
                updated_credits = current_credits - 1

                # Update the user's credits in the table
                update_response = (
                    self.supabase.table('credits')
                    .update({'credit_amount': updated_credits})
                    .eq('id', user_id)
                    .execute()
                )

                # Check if the update was successful
                if update_response.data:
                    self.logger.debug(f'Credit balance successfully decremented.')
                    return True
                else:
                    raise HTTPException(status_code=500, detail="Error decrementing credit balance.")

            # If user or credits not found, raise a 404 error
            raise HTTPException(status_code=404, detail="User data not found.")

        except Exception:
            # Handle exceptions (like if the user ID doesn't exist or there are database issues)
            raise HTTPException(status_code=500, detail="Error decrementing credit balance")

    def increment_credit_balance(self, user_id: str) -> bool:
        """
        Increments the user's credit balance by 1.

        Args:
            user_id (str): The ID of the current user.
        """
        try:
            # Query the public.credits table to get the user's current credits
            response = (
                self.supabase.table("credits")
                .select('*')
                .eq('id', user_id)
                .single()
                .execute()
            )
            # Check if the response was successful and contains data
            if response.data :
                current_credits = response.data['credit_amount']
                updated_credits = current_credits + 1

                # Update the user's credits in the table
                update_response = (
                    self.supabase.table('credits')
                    .update({'credit_amount': updated_credits})
                    .eq('id', user_id)
                    .execute()
                )

                # Check if the update was successful
                if update_response.data:
                    self.logger.debug(f'Credit balance successfully incremented.')
                    return True
                else:
                    raise HTTPException(status_code=500, detail="Error incrementing credit balance.")

            # If user or credits not found, raise a 404 error
            raise HTTPException(status_code=404, detail="User data not found.")

        except Exception:
            # Handle exceptions (like if the user ID doesn't exist or there are database issues)
            raise HTTPException(status_code=500, detail="Error incrementing credit balance")
